const { app, BrowserWindow , screen, globalShortcut,ipcMain} = require('electron')
const URL = require('url').URL
var path = require('path');
require("./env.js")
const {
  hasScreenCapturePermission,
  hasPromptedForPermission 
} = require('mac-screen-capture-permissions');

let win;
let primaryDisplay
let yOffset;
let pendingFullscreen;
let wasMinimized = false;
let normalWidth = 500;
let normalHeight = 700;
let restoredByCode = false;
let isWin = process.platform === "win32";

function createWindow () {
  primaryDisplay = screen.getPrimaryDisplay();
  yOffset = primaryDisplay.workArea.y + 1;
  maxWinHeight = primaryDisplay.workArea.height;


  console.log('Has asked permissions?', hasPromptedForPermission());

	console.log('Has permissions?', hasScreenCapturePermission());
	console.log('Has asked permissions?', hasPromptedForPermission());

  win = new BrowserWindow({
    x: primaryDisplay.workArea.width-normalWidth,
    y: primaryDisplay.bounds.height-normalHeight,
    width: normalWidth,
    height: normalHeight,
    titleBarStyle: 'hidden',
    frame: false,
    transparent: true,
    maximizable:false,
    resizable:false,
    webPreferences: {
      nodeIntegration: false, // is default value after Electron v5
      contextIsolation: true, // protect against prototype pollution
      enableRemoteModule: false, // turn off remote
      preload: path.join(__dirname, 'electronPreload.js'),
    }
  })
  
    win.webContents.session.setPermissionCheckHandler(async (webContents, permission, details) => {
      return true
    })
    win.webContents.session.setPermissionRequestHandler(async (webContents, permission, callback, details) => {
      callback(true)
    })
  
    if (process.env.NODE_ENV === 'development') {
      win.loadURL('https://localhost:3000/')
      console.log("loading url",'https://localhost:3000/',process.env.NODE_ENV)
    } else {
      win.loadURL('https://'+ ENV +'.icanread.io')
      console.log("loading url",'https://'+ ENV +'.icanread.io',process.env.NODE_ENV)
    }

    

  app.on('web-contents-created', (event, contents) => {
    contents.on('will-navigate', (event, navigationUrl) => {
      const regex = new RegExp('https://.*.icanread.io');
      const parsedUrl = new URL(navigationUrl)
      console.log(parsedUrl);
      if (!(regex.test(parsedUrl.origin))) {
        console.log("Prevented navigation to",parsedUrl);
        event.preventDefault()
      }
    })
  })
  app.on('web-contents-created', (event, contents) => {
    contents.setWindowOpenHandler(({ url }) => {
      return { action: 'deny' }
    })
  })
 
  screen.on("display-metrics-changed",(event,display,changedMetrics) => {
    console.log("Display metrics changed",display,primaryDisplay)
    if(primaryDisplay&&primaryDisplay.id!=screen.getPrimaryDisplay().id){
      primaryDisplay = screen.getPrimaryDisplay();
      if(!win.isMinimized()){
        ipcMain.emit('normalWindow');
      }
      win.reload();
      }
  })
  const ret = globalShortcut.register('CommandOrControl+I', () => {
    console.log('CommandOrControl+I is pressed')
    var point = screen.getCursorScreenPoint();
    console.log(`Mouse is at x: ${point.x} y: ${point.y}`);
    if(BrowserWindow.getAllWindows().length === 0){
      //On mac closing the window doesnt close the app. 
      //all windows closed. Create a new one so the user can read again.
      createWindow()
    } else {
      win.webContents.send('shortcutPressed', point, primaryDisplay,yOffset,maxWinHeight)
    }
  })


    // Event handler for asynchronous incoming messages
  ipcMain.on('fullScreenWindow', (event, args) => {
    console.log("on fullScreenWindow",args,Date.now());
    wasMinimized = win.isMinimized();
    if(isWin&&!wasMinimized){
      win.minimize();
      wasMinimized = true;
    }  
    console.log("wasMinimized",wasMinimized)

    
    if(wasMinimized){
      restoredByCode = true;
      win.restore();
    } 
    win.setHasShadow(false);
    if(isWin){
      // On Windows maximizing looks ok (no animation)
      win.maximize();
    } else {
      //Mac has ugly anaimation on maximize so resizing instead
      win.hide();
      win.setSize(primaryDisplay.workArea.width-1,primaryDisplay.workArea.height,false);
      win.setPosition(0,0,false);
      win.show();
    }
    
    win.webContents.send('fullScreenCompleted',"fromFullScreenWindow");
    console.log("sent fullScreenCompleted")
  })
  win.on("move",(event, args) => {
    console.log("win on move",Date.now());
    if(pendingFullscreen){
      pendingFullscreen = false;
      win.show();
      console.log("win on move pendingFullscreen",win.getContentSize(),Date.now());
      win.webContents.send('fullScreenCompleted',"fromMove")
    }
  }) 
  win.on("moved",(event, args) => {
    console.log("win on moved",Date.now());
    if(pendingFullscreen){
      pendingFullscreen = false;
      win.show();
      console.log("win on moved pendingFullscreen",win.getContentSize(),Date.now());
      win.webContents.send('fullScreenCompleted',"fromMoved")
    }
  })


// WM_EXITSIZEMOVE to make it work on Windows
if(win.hookWindowMessage){
  win.hookWindowMessage(Number.parseInt('0x0232'),(wParam,lParam)=>{
    console.log("Winows move or resize complete");
    if(pendingFullscreen){
      pendingFullscreen = false;
      win.show();
      console.log("win on moved pendingFullscreen",win.getContentSize());
      win.webContents.send('fullScreenCompleted',"fromWindowHook")
    }
  });
}


win.on("restore",() => {
  console.log("on restore")
  if(!restoredByCode){
    wasMinimized = false;
    win.setPosition(primaryDisplay.workArea.width-normalWidth,primaryDisplay.bounds.height-normalHeight,false);
    win.setSize(normalWidth,normalHeight,false);
  } else {
    restoredByCode = false;
  }

})

  // Event handler for asynchronous incoming messages
  ipcMain.on('normalWindow', (event, args) => {
    console.log("on normalWindow",args)
    win.hide()
    console.log("wasMinimized",wasMinimized)

    win.setSize(1,1,false);
    win.minimize()

  })

  if (!ret) {
    console.log('registration failed')
  }

  // Check whether a shortcut is registered.
  console.log(globalShortcut.isRegistered('CommandOrControl+i'))

  

}



app.disableHardwareAcceleration();
app.on('ready', () => setTimeout(createWindow, 300));


app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    globalShortcut.unregister('CommandOrControl+i')
    // Unregister all shortcuts.
    globalShortcut.unregisterAll()
    app.quit()
  }
})


app.on('certificate-error', function(event, webContents, url, error, 
  certificate, callback) {
      event.preventDefault();
      callback(true);
});

app.on('activate', () => {
  console.log("app.on activate")
  if (BrowserWindow.getAllWindows().length === 0) {

    createWindow()
  }
})